unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls, MoteurP4;

type
  TFMain = class(TForm)
    PanelPlateau: TPanel;
    ImageFond: TImage;
    ImageJeu: TImage;
    Timer: TTimer;
    PanelGeneral: TPanel;
    ButtonNouvellePartie: TButton;
    LabelJoueur1: TLabel;
    EditJoueur1: TEdit;
    EditJoueur2: TEdit;
    LabelJoueur2: TLabel;
    ImagePion1: TImage;
    ImagePion2: TImage;
    // FORM
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    // INTERFACE
    procedure ButtonNouvellePartieClick(Sender: TObject);
    procedure EditJoueurKeyUp(Sender: TObject; var Key: Word;  Shift: TShiftState);
    // INTERACTION GRAPHIQUE
    procedure TimerTimer(Sender: TObject);
    procedure ImageJeuClick(Sender: TObject);
    procedure ImageJeuMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  protected
    Plateau: TPlateau; // Plateau du jeu
    DDernierLache: TDateTime; // Time-stamp du dernier lach
  end;

var
  FMain: TFMain;

implementation

{$R *.dfm}

{ ============================================================================
  FORM
  ============================================================================ }
procedure TFMain.FormCreate(Sender: TObject);
begin
     // Initialisations
     Timer.Enabled := False;
     Plateau := TPlateau.Create( PanelPlateau,
                                 ImageFond.Picture.Bitmap, ImageJeu.Picture.Bitmap,
                                 ImagePion1.Picture.Bitmap, ImagePion2.Picture.Bitmap );
end;

procedure TFMain.FormDestroy(Sender: TObject);
begin
     Plateau.Free;
end;

{ ============================================================================
  INTERFACE
  ============================================================================ }
procedure TFMain.ButtonNouvellePartieClick(Sender: TObject);
begin
     Timer.Enabled := True;
     Plateau.NouveauJeu( EditJoueur1.Text, EditJoueur2.Text, Random( 2 ) + 1 );
end;

// Affectation nom du joueur
procedure TFMain.EditJoueurKeyUp(Sender: TObject; var Key: Word;  Shift: TShiftState);
begin
     if Assigned( Plateau ) then
       if Sender = EditJoueur1
         then Plateau.Joueur[ 1 ].SetNom( TEdit( Sender ).Text )
         else Plateau.Joueur[ 2 ].SetNom( TEdit( Sender ).Text )
end;

{ ============================================================================
  INTERACTIONS GRAPHIQUES
  ============================================================================ }

// Clignotement du jeton
procedure TFMain.TimerTimer(Sender: TObject);
begin
     //  Arret du timer si pas en cours de jeu
     if not Assigned( Plateau ) or ( Plateau.IJoueur = 0 )
       then Timer.Enabled := False
       else
       
     // si Sender = nil -> effacement forc
     if ( Plateau.IJoueur <> 0 ) and (Timer.Tag = 0 ) and  Assigned( Sender ) then
         begin
         Plateau.PionAfficher( Plateau.IJoueur, Point( Plateau.ICurseur, -1 ) );
         Timer.Tag := 1;
         Timer.Interval := 1000;
         end
       else
         begin
         Plateau.PionAfficher( 0, Point( Plateau.ICurseur, -1 ) );
         Timer.Tag := 0;
         Timer.Interval := 250;
         end;
end;

// Dplacement horinzontal du jeton  jouer
procedure TFMain.ImageJeuMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
   INewCurseur: Integer;
begin
     if Plateau.IJoueur = 0 then Exit; // pas en cours de jeu

     // nouvelle poisition
     INewCurseur := ( X - OFFSET.X ) div SEGMENT.X;
     if INewCurseur < 0 then INewCurseur := 0 else
     if INewCurseur > 6 then INewCurseur := 6;

     // effacement position prcdente
     if INewCurseur <> Plateau.ICurseur then
       if Timer.Enabled and ( Timer.Tag = 1 ) then
         begin
         TimerTimer( nil );
         Plateau.ICurseur := INewCurseur;
         TimerTimer( Self );
         end;
     Plateau.ICurseur := INewCurseur;
end;

// Jouer le jeton
procedure TFMain.ImageJeuClick(Sender: TObject);
begin
     if Plateau.IJoueur = 0 then Exit; // pas en cours de jeu
     if ( Now - DDernierLache ) < ( 0.1 / ( 3600 * 24 ) ) then Exit; // pour viter le multi lachs involontaire
     Plateau.Jouer;
     DDernierLache := Now;
end;


end.
