unit MoteurP4;

interface

uses Types, extCtrls, Graphics, Dialogs, SysUtils;

const
     // Paramtrages durs des bitmaps utiliss...
     SEGMENT: TPoint = ( x:40; y:40 ); // d'un trou  l'autre en pixels
     OFFSET: TPoint = ( x:16; y:40 ); // dcalage suprieur gauche de la grille en pixels
     JETON: TPoint = ( x:30; y:30 ); // dim. d'un jeton en pixels

     // incrmentation dans les 8 DIRECTIONS, regroupes 2 par 2 pour 4 SENS
     DIR_INCR : array[ 0..7] of TPoint = ( ( x:1;y:0),( x:-1;y:0),    // horizontale
                                           ( x:0;y:1),( x:0;y:-1),    // vertical
                                           ( x:1;y:1),( x:-1;y:-1),   // oblique slash
                                           ( x:-1;y:1),( x:1;y:-1) ); // oblique anti-slash

// CLASSE JOUEUR
type
  TPlateau = class;
  TJoueur = class( TObject )
  protected
    Plateau: TPlateau; // plateau propritaire
    SNom: string; // Nom du joueur
    ICamp: Integer; // camps (rouge ou jaune)
    IJetonsRestants : Integer; // nombre de jetons restants  jouer
  public
    // gestion
    constructor Create( zPlateau: TPlateau );
    destructor Destroy; override;
    procedure SetNom( zSNom: string ); // dfinition du nom du joueur
    // actions
    procedure Init( zSNom: string; zICamp: Integer ); // init gnral du joueur, son camp
    function Jouer( ICol: Integer ): Boolean; // Le joueur joue dans la colonne spcifie
  end;

// CLASSE PLATEAU
  TPlateau = class( TObject )
  protected
    // graphique
    Conteneur: TPanel; // conteneur visuel du plateau
    BmpFond, BmpJeu: TBitmap; // fond du jeu et grille
    BmpDoubleFond: TBitmap; // pour le gliss fluide du jeton
    BmpPion: array[ 1..2 ] of TBitmap; // dfinition des bitmaps des jetons
    // Plateau
    Grille: array[ 0..6, 0..5 ] of Integer; // grille virtuelle, contenu, Ndu camp (1/2) ou 0
    DernierJeton: TPoint; // coord du dernier jeton lach
    // Utilitaires
    function CaseVide( ICol: Integer ): Integer; // renvoie n de ligne dispo la plus basse
    function ResultatDernierJeton: Integer; // -1 nul, 0 en cours, 1 ou 2 gain
  public
    // Accs publique
    ICurseur: Integer; // colonne courante du jeton  jouer 0..6
    IJoueur: Integer; // Joueur en cours: 0->aucun, 1,2->numro du joueur
    Joueur: array[ 1..2 ] of TJoueur; // Dfinition des 2 joueurs
    // Crations
    constructor Create( zConteneur: TPanel; zBmpFond, zBmpJeu, zBmpPion1, zBmpPion2: TBitmap );
    destructor Destroy; override;
    // Actions
    procedure NouveauJeu( SNom1, SNom2: string; IPreums: Integer ); // lance une nouvelle partie
    procedure Jouer; // Joue un coup selon l'tat courant
    procedure Fin( SMessage: string ); // Fin de la partie + message
    // Interactions graphiques
    procedure PionAfficher( ICamp: Integer; PPos: TPoint; IDelta: Integer = 0 ); //afficher un jeton en coord logique
    procedure PionGlisser( ICamp, ICol, ILigneDest: Integer ); // chute d'une jeton
  end;


implementation

{ ============================================================================
  TJOUEUR
  ============================================================================ }
constructor TJoueur.Create( zPlateau: TPlateau );
begin
     Plateau := zPlateau;
     ICamp := 0; // n'appartient  aucun camps
     IJetonsRestants := 0;
end;

destructor TJoueur.Destroy;
begin
     //
     inherited;
end;

procedure TJoueur.SetNom( zSNom: string );
begin
     if zSNom <> ''
       then SNom := zSNom;
end;

procedure TJoueur.Init( zSNom: string; zICamp: Integer );
begin
     SetNom( zSNom );
     ICamp := zICamp;
     IJetonsRestants := 21;
end;

function TJoueur.Jouer( ICol: Integer ): Boolean;
var
   ILigne: Integer;
begin
     Result := False;
     ILigne := Plateau.CaseVide( ICol );
     if ILigne <> -1 then
       begin
       Plateau.Grille[ ICol, ILigne ] := ICamp;
       Plateau.DernierJeton := Point( ICol, ILigne );
       Dec( IJetonsRestants );
       Result := True;
       end;
end;

{ ============================================================================
  TPLATEAU
  ============================================================================ }

// Cration du plateau, fonds bitmaps et joueurs. La partie est arrete par dfaut
constructor TPlateau.Create( zConteneur: TPanel; zBmpFond, zBmpJeu, zBmpPion1, zBmpPion2: TBitmap );
begin
     // var
     Conteneur := zConteneur;
     Conteneur.DoubleBuffered := True;
     BmpFond := zBmpFond;
     BmpJeu := zBmpJeu;
     BmpPion[ 1 ] := zBmpPion1;
     BmpPion[ 2 ] := zBmpPion2;
     // Cration des Joueurs 1 et 2
     Joueur[ 1 ] := TJoueur.Create( Self );
     Joueur[ 2 ] := TJoueur.Create( Self );
     // inits
     BmpDoubleFond := TBitmap.Create;
     BmpDoubleFond.Assign( BmpFond );
     ICurseur := 3;
end;

// Restitution
destructor TPlateau.Destroy;
begin
     Joueur[ 1 ].Free;
     Joueur[ 2 ].Free;
     BmpDoubleFond.FreeImage;
     BmpDoubleFond.Free;
     inherited;
end;

// Initialisations d'un nouveau Jeu.
// IPreums: N du joueur qui commence
procedure TPlateau.NouveauJeu( SNom1, SNom2: string; IPreums: Integer );
var
   x, y: Integer;
begin
     for x := 0 to 6 do
       for y := 0 to 5
         do Grille[ x, y ] := 0;
     BmpFond.Assign( BmpDoubleFond );
     Joueur[ 1 ].Init( SNom1, 1 );
     Joueur[ 2 ].Init( SNom2, 2 );
     IJoueur := IPreums;
end;

// Jouer un jeton, selon la colonne courante et le joueur courant
procedure TPlateau.Jouer;
var
   IEtat: Integer;
begin
     // Fait jouer le joueur actuel
     if Joueur[ IJoueur ].Jouer( ICurseur ) then
       begin
       PionGlisser( Joueur[ IJoueur ].ICamp, DernierJeton.X, DernierJeton.Y );
       IEtat := ResultatDernierJeton;
       // Rsultat du coup - -1 / 0 / 1 ou 2
       case IEtat of
         -1  : Fin( 'PARTIE NULLE !' );
         1,2 : Fin( Joueur[ IEtat ].SNom + #13 + #10 + 'is the winner !' );
         else  IJoueur := ( IJoueur mod 2 ) + 1;
       end;
       end;

end;

// Message de fin de jeu
procedure TPlateau.Fin( SMessage: string );
begin
     IJoueur := 0;
     ShowMessage( SMessage );
end;

// dtection de la dernire cellule vide en partant du haut dans la col. spcifie
function TPlateau.CaseVide( ICol: Integer ): Integer;
var
   i: Integer;
begin
     Result := -1;
     for i := 0 to 5 do
       if Grille[ ICol, i ] = 0
         then Result := i
         else Break;
end;

// Analyse de l'tat du jeu, uniquement en fonction du dernier jeton lach
// Retour: 0=RAS la partie continue, -1=partie nulle, 1 ou 2=N du joueur gagnant
function TPlateau.ResultatDernierJeton: Integer;
var
   i, j: Integer;
   TabDirCompteur: array[ 0..7 ] of Integer; // Compteur par direction
   TabDirOK: array[ 0..7 ] of Boolean; // Directions  exploiter, False: ne plus exploiter
   xi, yi: Integer;
begin
     // Inits des compteurs
     Result := 0;
     for i := 0 to 7 do
       begin
       TabDirCompteur[ i ] := 0; // 0 jeton conscutif
       TabDirOK[ i ] := True; // direction exploitable
       end;

     // Comptage des jetons conscutifs dans les 8 directions
     for i := 0 to 7 do // les 8 directions = 8 jetons potentiels autour
       for j := 1 to 3 do // 3 niveaux conscutifs + jeton central = test de 4 jetons aligns potentiels
         if TabDirOK[ i ] then // Direction exploitable ?
           begin
           // coord du jeton voisin
           xi := DernierJeton.X + DIR_INCR[ i ].X * j;
           yi := DernierJeton.Y + DIR_INCR[ i ].Y * j;
           if ( xi < 0 ) or ( xi > 6 ) or ( yi < 0 ) or ( yi > 5 ) then Continue; // hors du jeu
           // Cas du jeton voisin = jeton central -> incrmentation du sens en cours
           if Grille[ xi, yi ] = Grille[ DernierJeton.X, DernierJeton.Y ]
             then Inc( TabDirCompteur[ i ] )
             else TabDirOK[ i ] := False; // au 1er jeton diffrent, la direction n'est plus  exploiter
           end;

     // Bilan des compteurs, dans les 4 sens, 2 par 2 directions selon DIR_INCR
     for i := 0 to 3 do
       if TabDirCompteur[ i*2 ] + TabDirCompteur[ i*2+1 ] >= 3 // il peut y avoir plus de 4 jetons aligns...
         then Result := IJoueur;

     // Partie Nulle ?
     if Result = 0 then
       if ( Joueur[ 1 ].IJetonsRestants = 0 ) and ( Joueur[ 2 ].IJetonsRestants = 0 )
         then Result := -1;
end;

// Affichage/effacement d'un jeton sur le plateau
procedure TPlateau.PionAfficher( ICamp: Integer; PPos: TPoint; IDelta: Integer = 0 );
var
   RRect: TRect;
   BmpSource: TBitmap;
begin
     // zone de dessin
     RRect.Left := OFFSET.X + SEGMENT.X * PPos.X;
     RRect.Top := OFFSET.Y + SEGMENT.Y * PPos.Y + IDelta;
     RRect.Right := RRect.Left + JETON.X;
     RRect.Bottom := RRect.Top + JETON.Y;
     // Type de dessin
     if ICamp > 0
       then BmpSource := BmpPion[ ICamp ]
       else BmpSource := Nil;
     // Affichage du dessin
     if Assigned( BmpSource )
       then BmpFond.Canvas.Draw( RRect.Left, RRect.Top, BmpSource ) // affichage Jeton
       else BmpFond.Canvas.CopyRect( RRect, BmpDoubleFond.Canvas, RRect ); // restitution du fond
end;

// Glissement d'un jeton sur le plateau
procedure TPlateau.PionGlisser( ICamp, ICol, ILigneDest: Integer );
var
   IDelta: Integer;
   RInc: Double;
   IDeltaMax: Integer;
begin
     // Inits
     IDelta := 0;
     RInc := 0;
     IDeltaMax := ILigneDest * SEGMENT.Y + JETON.Y div 2;

     // dbut
     PionAfficher( ICamp, Point( ICol, -1 ), 0 );
     Conteneur.Refresh;
     Sleep( 100 );

     // glissement
     while IDelta < IDeltaMax do
       begin
       PionAfficher( 0, Point( ICol, -1 ), IDelta );
       Inc( IDelta, Round( RInc ) );
       PionAfficher( ICamp, Point( ICol, -1 ), IDelta );
       RInc := RInc + 0.6; // acclration approximative du jeton
       Conteneur.Refresh;
       Sleep( 8 );
       end;
       
     // fin
     PionAfficher( 0, Point( ICol, -1 ), IDelta ); // effacement jeton  jouer
     PionAfficher( ICamp, Point( ICol, ILigneDest ), 0 ); // jeton position finale
     Conteneur.Refresh;
end;


end.
 